import asyncio
from . import BotCheckin

from pyrogram.types import Message
from pyrogram.errors import ChatWriteForbidden

__ignore__ = True


class EPubGroupCheckin(BotCheckin):
    name = "EPub 电子书库群组签到"
    chat_name = "libhsulife"
    bot_username = "zhruonanbot"
    bot_use_captcha = False

    async def send_checkin(self, retry=False):
        try:
            msg = await self.send("签到")
            if msg:
                self.mid = msg.id
        except ChatWriteForbidden:
            self.log.info("被禁言, 准备 2 分钟后重新签到.")
            await asyncio.sleep(120)
            await self.retry()

    async def message_handler(self, client, message: Message, type=None):
        mid = getattr(self, "mid", None)
        if mid and message.reply_to_message_id == mid:
            return await super().message_handler(client, message, type=type)



import asyncio
from abc import ABC, abstractmethod

from loguru import logger

from embykeeper.runinfo import RunContext, RunStatus
from embykeeper.utils import show_exception
from embykeeper.config import config
from embykeeper.telegram.pyrogram import Client

__ignore__ = True

logger = logger.bind(scheme="teleregistrar")


class BaseBotRegister(ABC):
    """基础注册类."""

    name: str = None

    def __init__(
        self,
        client: Client,
        context: RunContext = None,
        retries=None,
        timeout=None,
        config: dict = {},
    ):
        self.client = client
        self.ctx = context or RunContext.prepare()

        self._retries = retries
        self._timeout = timeout

        self.config = config
        self.finished = asyncio.Event()  # 注册完成事件
        self.log = self.ctx.bind_logger(logger.bind(name=self.name, username=client.me.full_name))  # 日志组件

        self._task = None  # 主任务

    @property
    def retries(self):
        return self._retries or getattr(config, "register", {}).get("retries", 1)

    @property
    def timeout(self):
        return self._timeout or getattr(config, "register", {}).get("timeout", 120)

    async def _start(self):
        """注册器的入口函数的错误处理外壳."""
        try:
            self.client.stop_handlers.append(self.stop)
            self._task = asyncio.create_task(self.start())
            return await self._task
        except Exception as e:
            if config.nofail:
                self.log.warning(f"初始化异常错误, 注册器将停止.")
                show_exception(e, regular=False)
                return self.ctx.finish(RunStatus.ERROR, "异常错误")
            else:
                raise
        finally:
            if hasattr(self.client, "stop_handlers") and self.stop in self.client.stop_handlers:
                self.client.stop_handlers.remove(self.stop)
            self._task = None

    @abstractmethod
    async def start(self) -> RunContext:
        pass

    async def stop(self):
        if self._task:
            self._task.cancel()
            try:
                await self._task
            except asyncio.CancelledError:
                pass



from json import JSONDecodeError
from datetime import datetime

from curl_cffi.requests import AsyncSession, RequestsError, Response

from embykeeper.runinfo import RunStatus
from embykeeper.utils import get_proxy_str, show_exception
from embykeeper.config import config

from . import BotCheckin

__ignore__ = True


class DPeakCheckin(BotCheckin):
    name = "DPeak"
    bot_username = "emby_dpeak_bot"
    additional_auth = ["prime"]

    async def send_checkin(self, **kw):
        try:
            async with AsyncSession(
                proxy=get_proxy_str(config.proxy, curl=True), impersonate="edge", allow_redirects=True
            ) as session:
                tgid = self.client.me.id
                current_time = datetime.now()
                date_str = current_time.strftime("%Y-%m-%d")

                # Get info
                resp: Response = await session.get(f"https://miniapp.bwihz.cn/records/checkins/{tgid}.json")
                if resp.ok:
                    result = resp.json()
                    last_checkin = result["lastCheckIn"]
                    streak = result["streak"]
                    checkin_history = result.get("checkInHistory", {})
                else:
                    last_checkin = date_str
                    streak = 0
                    checkin_history = {}

                # First request to get token
                resp: Response = await session.post(
                    "https://miniapp.bwihz.cn/api/token.php", json={"tgid": tgid, "action": "checkin"}
                )
                token_data = resp.json()
                token = token_data["token"]

                # Second request to perform check-in
                timestamp = int(current_time.timestamp())
                iso_time = current_time.strftime("%Y-%m-%dT%H:%M:%S.%f")[:-3] + "+08:00"

                checkin_data = {
                    "tgid": tgid,
                    "checkInData": {
                        "lastCheckIn": last_checkin,
                        "streak": streak + 1,
                        "checkInHistory": checkin_history,
                    },
                    "clientTime": iso_time,
                    "clientTimestamp": timestamp,
                    "token": token,
                }
                resp = await session.post(
                    "https://miniapp.bwihz.cn/api/checkin_record.php", json=checkin_data
                )
                result = resp.json()

                if result.get("success"):
                    details = result["checkInDetails"]
                    add_points = details["totalReward"]
                    self.log.info(f"[yellow]签到成功[/]: + {add_points} 分.")
                    return await self.finish(RunStatus.SUCCESS, "签到成功")
                else:
                    error = result.get("error", None)
                    if error:
                        if "签到过了" in error:
                            self.log.info(f"今日已经签到过了.")
                            return await self.finish(RunStatus.NONEED, "今日已签到")
                        self.log.info(f"签到失败: {error}.")
                    else:
                        self.log.info(f"签到失败, 请求状态为 {resp.status_code}:\n{result}.")
                    return await self.retry()

        except (RequestsError, OSError, JSONDecodeError) as e:
            self.log.info(f"签到失败: 无法连接签到页面 ({e.__class__.__name__}: {e}).")
            return await self.retry()



import asyncio
import random
import string
from typing import Optional

from pydantic import BaseModel, ValidationError

from embykeeper.telegram.embyboss import EmbybossRegister
from embykeeper.runinfo import RunStatus

from . import BaseBotRegister

__ignore__ = True


class TemplateARegisterConfig(BaseModel):
    name: Optional[str] = None
    username: Optional[str] = None  # 可选, 不填则自动生成
    password: Optional[str] = None  # 可选, 不填则自动生成


class TemplateARegister(BaseBotRegister):
    def __init__(self, client, context=None, retries=None, timeout=None, config=None, **kwargs):
        # 设置基本属性
        self.bot_username = kwargs.get("bot_username")
        self.name = kwargs.get("name", f"@{self.bot_username}")
        self.templ_name = kwargs.get("templ_name")

        # 调用父类构造函数
        super().__init__(client, context, retries, timeout, config)

        try:
            self.t_config = TemplateARegisterConfig.model_validate(self.config)
        except ValidationError as e:
            self.log.warning(f"初始化失败: 注册自定义模板 A 的配置错误:\n{e}")
            self.t_config = TemplateARegisterConfig()  # 使用默认配置

    async def start(self):
        if not self.bot_username:
            self.log.warning("未配置bot_username, 无法进行注册")
            return self.ctx.finish(RunStatus.FAIL, "未配置bot_username")

        for attempt in range(self.retries + 1):
            try:
                username = self.t_config.username or self.client.me.username or f"user_{self.client.me.id}"
                password = self.t_config.password or "".join(
                    random.choices(string.ascii_letters + string.digits, k=4)
                )

                embyboss_register = EmbybossRegister(
                    client=self.client, logger=self.log, username=username, password=password
                )

                # 使用超时控制
                success = await asyncio.wait_for(
                    embyboss_register.run(self.bot_username), timeout=self.timeout
                )

                if success:
                    self.log.info("注册成功")
                    return self.ctx.finish(RunStatus.SUCCESS, "注册成功")
                else:
                    if attempt < self.retries:
                        self.log.warning(f"注册失败, 将进行第 {attempt + 2} 次重试")
                        await asyncio.sleep(1)  # 重试间隔
                        continue
                    else:
                        self.log.warning("注册失败, 已达到最大重试次数")
                        return self.ctx.finish(RunStatus.FAIL, "注册失败")

            except asyncio.TimeoutError:
                if attempt < self.retries:
                    self.log.warning(f"注册超时 ({self.timeout}秒), 将进行第 {attempt + 2} 次重试")
                    await asyncio.sleep(1)  # 重试间隔
                    continue
                else:
                    self.log.error(f"注册超时, 已达到最大重试次数")
                    return self.ctx.finish(RunStatus.FAIL, f"注册超时 ({self.timeout}秒)")
            except Exception as e:
                if attempt < self.retries:
                    self.log.warning(f"注册异常: {e}, 将进行第 {attempt + 2} 次重试")
                    await asyncio.sleep(1)  # 重试间隔
                    continue
                else:
                    self.log.error(f"注册过程中发生异常: {e}, 已达到最大重试次数")
                    return self.ctx.finish(RunStatus.FAIL, f"注册异常: {e}")

        return self.ctx.finish(RunStatus.FAIL, "注册失败")


def use(**kw):
    return type("TemplatedRegisterClass", (TemplateARegister,), kw)



from pyrogram.types import Message

from ._templ_a import TemplateACheckin


class DPeakCheckin(TemplateACheckin):
    name = "DPeak"
    bot_username = "emby_dpeak_bot"
    additional_auth = ["prime"]

    async def message_handler(self, client, message: Message):
        if message.text and "人机验证" in message.text:
            if not await self.gpt_handle_message(message, unexpected=False):
                self.log.info(f"签到失败: 智能解析错误, 正在重试.")
                return await self.retry()
            else:
                return
        await super().message_handler(client, message)



from .pornfans_alert import PornfansAlertMonitor

__ignore__ = True


class TestPornfansAlertMonitor(PornfansAlertMonitor):
    name = "PornFans 风险急停 测试"
    chat_name = "api_group"
    chat_allow_outgoing = True



from pyrogram.types import Message

from . import Monitor

__ignore__ = True


class TestPornfansMonitor(Monitor):
    name = "PornFans 消息接收 测试"
    chat_name = "Porn_Emby_Bot"
    chat_keyword = r".*"

    async def on_trigger(self, message: Message, key, reply):
        self.log.info(f"PornFans 消息接收 测试: {message.text or message.caption}")



import asyncio
import random
from typing import List, Optional, Union

from loguru import logger
from pydantic import BaseModel, ValidationError
from pyrogram.types import Message
from pyrogram.errors import MessageIdInvalid
from pyrogram.raw.types.messages import BotCallbackAnswer

from embykeeper.utils import to_iterable

from . import BotCheckin

__ignore__ = True


class TemplateACheckinConfig(BaseModel):
    # fmt: off
    name: Optional[str] = None  # 签到器的名称
    use_button_answer: bool = None  # 点击按钮后等待并识别响应
    bot_text_ignore_answer: Union[str, List[str]] = None  # 忽略的响应文本
    bot_fail_keywords: Union[str, List[str]] = None  # 签到错误将重试时检测的关键词 (暂不支持regex), 置空使用内置关键词表
    bot_success_keywords: Union[str, List[str]] = None  # 成功时检测的关键词 (暂不支持regex), 置空使用内置关键词表
    bot_success_pat: Optional[str] = None  # 当接收到成功消息后, 从消息中提取数字的模式
    bot_captcha_len: Optional[int] = None  # 验证码长度的可能范围
    bot_text_ignore: Union[str, List[str]] = None  # 当含有列表中的关键词, 即忽略该消息, 置空不限制
    bot_checkin_caption_pat: Optional[str] = None  # 当 Bot 返回图片时, 仅当符合该 regex 才识别为验证码, 置空不限制
    bot_checkin_cmd: Optional[str] = None  # Bot 依次执行的签到命令
    bot_use_captcha: Optional[bool] = None  # 当 Bot 返回图片时, 识别验证码并调用 on_captcha
    bot_checkin_button: Union[str, List[str]] = None  # 签到按钮文本
    templ_panel_keywords: Union[str, List[str]] = None  # 面板关键词
    # fmt: on


class TemplateACheckin(BotCheckin):
    bot_text_ignore_answer = ["Done"]
    use_button_answer = True
    bot_checkin_cmd = "/start"
    bot_checkin_button = ["签到", "簽到"]
    templ_panel_keywords = None

    async def init(self):
        try:
            self.t_config = TemplateACheckinConfig.model_validate(self.config)
        except ValidationError as e:
            self.log.warning(f"初始化失败: 签到自定义模板 A 的配置错误:\n{e}")
            return False

        self.name = self.t_config.name or self.name
        self.use_button_answer = (
            self.t_config.use_button_answer
            if self.t_config.use_button_answer is not None
            else self.use_button_answer
        )
        self.bot_text_ignore_answer = self.t_config.bot_text_ignore_answer or self.bot_text_ignore_answer
        self.bot_fail_keywords = self.t_config.bot_fail_keywords or self.bot_fail_keywords
        self.bot_success_keywords = self.t_config.bot_success_keywords or self.bot_success_keywords
        self.bot_success_pat = self.t_config.bot_success_pat or self.bot_success_pat
        self.bot_captcha_len = self.t_config.bot_captcha_len or self.bot_captcha_len
        self.bot_text_ignore = self.t_config.bot_text_ignore or self.bot_text_ignore
        self.bot_checkin_caption_pat = self.t_config.bot_checkin_caption_pat or self.bot_checkin_caption_pat
        self.bot_checkin_cmd = self.t_config.bot_checkin_cmd or self.bot_checkin_cmd
        self.bot_checkin_button = self.t_config.bot_checkin_button or self.bot_checkin_button
        self.templ_panel_keywords = self.t_config.templ_panel_keywords or self.templ_panel_keywords
        self.bot_use_captcha = (
            self.t_config.bot_use_captcha
            if self.t_config.bot_use_captcha is not None
            else self.bot_use_captcha
        )

        self.log = logger.bind(scheme="telechecker", name=self.name, username=self.client.me.full_name)
        return True

    async def message_handler(self, client, message: Message):
        text = message.caption or message.text
        if (
            text
            and message.reply_markup
            and (
                (
                    self.templ_panel_keywords
                    and any(keyword in text for keyword in to_iterable(self.templ_panel_keywords))
                )
                or (getattr(message, "is_first_response", False) and not message.edit_date)
            )
        ):
            keys = [k.text for r in message.reply_markup.inline_keyboard for k in r]
            for k in keys:
                if any(btn in k for btn in self.bot_checkin_button):
                    await asyncio.sleep(random.uniform(0.5, 1.5))
                    try:
                        answer: BotCallbackAnswer = await message.click(k)
                    except TimeoutError:
                        self.log.debug(f"点击签到按钮无响应, 可能按钮未正确处理点击回复. 一般来说不影响签到.")
                    except MessageIdInvalid:
                        pass
                    else:
                        await self.on_button_answer(answer)
                    return
            else:
                self.log.warning(f"签到失败: 账户错误.")
                return await self.fail()

        if message.text and "请先点击下面加入我们的" in message.text:
            self.log.warning(f"签到失败: 账户错误.")
            return await self.fail()

        await super().message_handler(client, message)

    async def on_button_answer(self, answer: BotCallbackAnswer):
        if self.use_button_answer:
            if not isinstance(answer, BotCallbackAnswer):
                self.log.warning(f"签到失败: 签到按钮指向 URL, 不受支持.")
                return await self.fail()
            if answer.message and not any(ignore in answer.message for ignore in self.bot_text_ignore_answer):
                await self.on_text(Message(id=0, text=answer.message), answer.message)


def use(**kw):
    return type("TemplatedClass", (TemplateACheckin,), kw)



import asyncio
from datetime import datetime, time

from ..lock import (
    pornfans_nohp,
    pornfans_messager_enabled,
    pornfans_messager_mids,
    pornfans_messager_mids_lock,
    pornfans_alert,
)
from . import Messager

__ignore__ = True


class PornfansMessager(Messager):
    name = "PornFans"
    chat_name = "embytestflight"
    default_messages = ["pornemby-common-wl@latest.yaml * 100"]

    async def init(self):
        self.lock = asyncio.Lock()
        pornfans_messager_enabled[self.me.id] = True
        async with pornfans_messager_mids_lock:
            if self.me.id not in pornfans_messager_mids:
                pornfans_messager_mids[self.me.id] = []
        return True

    async def send(self, message):
        if pornfans_alert.get(self.me.id, False):
            self.log.info(f"由于风险急停取消发送.")
            return
        nohp_date = pornfans_nohp.get(self.me.id, None)
        if nohp_date and nohp_date >= datetime.today().date():
            self.log.info(f"取消发送: 血量已耗尽.")
            return
        msg = await super().send(message)
        if msg:
            pornfans_messager_mids[self.me.id].append(msg.id)
        return msg



from .pornfans_dragon_rain import PornfansDragonRainMonitor

__ignore__ = True


class TestPornfansDragonRainMonitor(PornfansDragonRainMonitor.PornfansDragonRainClickMonitor):
    name = "PornFans 红包雨 测试"
    chat_name = "api_group"
    chat_user = "embykeeper_test_bot"
    chat_allow_outgoing = True



from ._templ_a import TemplateACheckin

from pyrogram.types import Message

__ignore__ = True


class BibiCheckin(TemplateACheckin):
    name = "比比"
    bot_username = "BBFreeFilm_bot"
    bot_use_captcha = False

    async def message_handler(self, client, message: Message):
        text = message.text or message.caption
        if text and "签到验证" in text:
            if "开始" in text:
                return
            if not await self.gpt_handle_message(message, unexpected=False):
                self.log.info(f"签到失败: 智能解析错误, 正在重试.")
                return await self.retry()
            else:
                return
        await super().message_handler(client, message)



from embykeeper.runinfo import RunStatus

from . import BotCheckin

__ignore__ = True


class HDHiveCheckin(BotCheckin):
    name = "HDHive"
    bot_username = "HDHiveBot"
    bot_account_fail_keywords = ["你需要先完成用户初始化"]

    async def send_checkin(self, retry=False):
        await self.send("/checkin")
        await self.finish(RunStatus.SUCCESS, "已发送签到命令")



from __future__ import annotations

import base64
from collections import OrderedDict
from contextlib import asynccontextmanager
from datetime import datetime
import asyncio
import inspect
import os
from pathlib import Path
import sqlite3
import struct
import tempfile
from typing import Union
import logging

from rich.prompt import Prompt
from loguru import logger
import pyrogram
from pyrogram import raw, types, filters, dispatcher
from pyrogram.enums import SentCodeType
from pyrogram.errors import (
    BadRequest,
    SessionPasswordNeeded,
    CodeInvalid,
    PhoneCodeInvalid,
    FloodWait,
    PhoneNumberInvalid,
    PhoneNumberBanned,
    MessageIdInvalid,
)
from pyrogram.handlers import (
    MessageHandler,
    RawUpdateHandler,
    DisconnectHandler,
    EditedMessageHandler,
    StartHandler,
    StopHandler,
    ConnectHandler,
)
from pyrogram.storage.sqlite_storage import SQLiteStorage, TEST, PROD
from pyrogram.handlers.handler import Handler

from embykeeper import var, __name__ as __product__, __version__
from embykeeper.schema import TelegramAccount
from embykeeper.utils import async_partial, show_exception

var.tele_used.set()

logger = logger.bind(scheme="telegram", nonotify=True)


class LogRedirector(logging.StreamHandler):
    def emit(self, record):
        try:
            if record.levelno >= logging.WARNING:
                logger.debug(f"Pyrogram 警告: {record.getMessage()}")
        except (KeyboardInterrupt, SystemExit):
            raise
        except:
            self.handleError(record)


pyrogram_session_logger = logging.getLogger("pyrogram")
for h in pyrogram_session_logger.handlers[:]:
    pyrogram_session_logger.removeHandler(h)
pyrogram_session_logger.addHandler(LogRedirector())


class Dispatcher(dispatcher.Dispatcher):
    updates_count = 0

    def __init__(self, client: Client):
        super().__init__(client)
        self.mutex = asyncio.Lock()

    async def start(self):
        phone_masked = TelegramAccount.get_phone_masked(self.client.phone_number)
        logger.debug(f'Telegram 更新分配器正在启动: "{phone_masked}".')

        if callable(self.client.start_handler):
            try:
                await self.client.start_handler(self.client)
            except Exception as e:
                show_exception(e, regular=False)
                logger.error("Telegram 更新分配器启动错误.")

        if not self.client.no_updates:
            for _ in range(self.client.workers):
                self.handler_worker_tasks.append(self.client.loop.create_task(self.handler_worker()))

            if not self.client.skip_updates:
                await self.client.recover_gaps()

        logger.debug(f'Telegram 更新分配器已启动: "{phone_masked}".')

    async def stop(self, clear_handlers: bool = True):
        phone_masked = TelegramAccount.get_phone_masked(self.client.phone_number)
        logger.debug(f'Telegram 更新分配器正在停止: "{phone_masked}".')

        if callable(self.client.stop_handler):
            try:
                await self.client.stop_handler(self.client)
            except Exception as e:
                show_exception(e, regular=False)
                logger.error("Telegram 更新分配器停止错误.")

        if not self.client.no_updates:
            for i in range(self.client.workers):
                self.updates_queue.put_nowait(None)

            for i in self.handler_worker_tasks:
                i.cancel()
                try:
                    await i
                except asyncio.CancelledError:
                    pass
            if clear_handlers:
                self.handler_worker_tasks.clear()
                self.groups.clear()

        logger.debug(f'Telegram 更新分配器已停止: "{phone_masked}".')

    def add_handler(self, handler, group: int):
        async def fn():
            async with self.mutex:
                if group not in self.groups:
                    self.groups[group] = []
                    self.groups = OrderedDict(sorted(self.groups.items()))
                self.groups[group].append(handler)
                # logger.debug(f"增加了 Telegram 更新处理器: {handler.__class__.__name__}.")

        return self.client.loop.create_task(fn())

    def remove_handler(self, handler, group: int):
        async def fn():
            async with self.mutex:
                if group not in self.groups:
                    raise ValueError(f"Group {group} does not exist. Handler was not removed.")
                self.groups[group].remove(handler)
                # logger.debug(f"移除了 Telegram 更新处理器: {handler.__class__.__name__}.")

        return self.client.loop.create_task(fn())

    async def handler_worker(self):
        while True:
            packet = await self.updates_queue.get()
            Dispatcher.updates_count += 1

            if packet is None:
                break

            try:
                update, users, chats = packet
                parser = self.update_parsers.get(type(update), None)

                try:
                    parsed_update, handler_type = (
                        await parser(update, users, chats) if parser is not None else (None, type(None))
                    )
                except (ValueError, BadRequest) as e:
                    logger.warning(f"更新处理器发生错误, 可能遗漏消息.")
                    show_exception(e, regular=False)
                    continue

                async with self.mutex:
                    groups = {i: g[:] for i, g in self.groups.items()}

                for group in groups.values():
                    for handler in group:
                        args = None

                        if isinstance(handler, handler_type):
                            try:
                                if await handler.check(self.client, parsed_update):
                                    args = (parsed_update,)
                            except Exception as e:
                                logger.warning(f"更新处理器发生错误, 可能遗漏消息.")
                                show_exception(e, regular=False)
                                continue

                        elif isinstance(handler, RawUpdateHandler):
                            try:
                                if await handler.check(self.client, update):
                                    args = (update, users, chats)
                            except Exception as e:
                                logger.warning(f"更新处理器发生错误, 可能遗漏消息.")
                                show_exception(e, regular=False)
                                continue

                        if args is None:
                            continue

                        try:
                            if inspect.iscoroutinefunction(handler.callback):
                                await handler.callback(self.client, *args)
                            else:
                                await self.client.loop.run_in_executor(
                                    self.client.executor, handler.callback, self.client, *args
                                )
                        except pyrogram.StopPropagation:
                            raise
                        except pyrogram.ContinuePropagation:
                            continue
                        except Exception as e:
                            logger.error(f"更新回调函数内发生错误.")
                            show_exception(e, regular=False)
                        break
                    else:
                        continue
                    break
            except pyrogram.StopPropagation:
                pass
            except Exception as e:
                logger.warning("更新控制器错误.")
                show_exception(e, regular=False)


class FileStorage(SQLiteStorage):
    async def open(self):
        path = self.database
        file_exists = path.is_file()

        # Try to create database in the original path
        try:
            # Ensure parent directory exists
            path.parent.mkdir(parents=True, exist_ok=True)
            self.conn = sqlite3.connect(str(path), timeout=1, check_same_thread=False)
        except sqlite3.OperationalError as e:
            if "unable to open database file" in str(e):
                # Test write permission by trying to create a temporary file
                test_error = None
                try:
                    test_file = path.parent / f".{self.name}_test"
                    test_file.write_text("test")
                    test_file.unlink()
                except Exception as write_e:
                    test_error = str(write_e)

                error_msg = f"无法在默认路径创建数据库文件 {path}: {e}"
                if test_error:
                    error_msg += f" (写入测试失败: {test_error})"
                else:
                    error_msg += " (目录可写, 可能是 SQLite 特定问题)"

                logger.warning(error_msg)

                # Fallback to system temp directory
                temp_dir = Path(tempfile.gettempdir())
                temp_path = temp_dir / (self.name + self.FILE_EXTENSION)

                logger.info(f"尝试在临时目录创建会话文件: {temp_path}")

                try:
                    self.conn = sqlite3.connect(str(temp_path), timeout=1, check_same_thread=False)
                    # Update the database path to the new location
                    self.database = temp_path
                    path = temp_path
                    file_exists = temp_path.is_file()
                    logger.info(f"成功在临时目录创建会话文件: {temp_path}")
                except sqlite3.OperationalError as temp_e:
                    logger.error(f"无法创建数据库文件: {temp_e}")
                    raise temp_e
            else:
                raise

        if self.use_wal:
            self.conn.execute("PRAGMA journal_mode=WAL")
        else:
            self.conn.execute("PRAGMA journal_mode=DELETE")

        # Check if database has required tables before calling update
        database_is_valid = False
        if file_exists:
            try:
                # Try to check if version table exists
                cursor = self.conn.cursor()
                cursor.execute("SELECT name FROM sqlite_master WHERE type='table' AND name='version';")
                if cursor.fetchone():
                    database_is_valid = True
                cursor.close()
            except sqlite3.Error:
                # Database is corrupted or incomplete
                database_is_valid = False

        if not file_exists or not database_is_valid:
            if file_exists and not database_is_valid:
                logger.debug(f"数据库文件结构不完整, 重新初始化: {path}")
            await self.create()
            if self.session_string:
                # Old format
                if len(self.session_string) in [self.SESSION_STRING_SIZE, self.SESSION_STRING_SIZE_64]:
                    dc_id, test_mode, auth_key, user_id, is_bot = struct.unpack(
                        (
                            self.OLD_SESSION_STRING_FORMAT
                            if len(self.session_string) == self.SESSION_STRING_SIZE
                            else self.OLD_SESSION_STRING_FORMAT_64
                        ),
                        base64.urlsafe_b64decode(self.session_string + "=" * (-len(self.session_string) % 4)),
                    )

                    await self.dc_id(dc_id)
                    await self.test_mode(test_mode)
                    await self.auth_key(auth_key)
                    await self.user_id(user_id)
                    await self.is_bot(is_bot)
                    await self.date(0)

                    logger.warning(
                        "You are using an old session string format. Use export_session_string to update"
                    )
                    return

                dc_id, api_id, test_mode, auth_key, user_id, is_bot = struct.unpack(
                    self.SESSION_STRING_FORMAT,
                    base64.urlsafe_b64decode(self.session_string + "=" * (-len(self.session_string) % 4)),
                )

                await self.dc_id(dc_id)

                if test_mode:
                    await self.server_address(TEST[dc_id])
                    await self.port(80)
                else:
                    await self.server_address(PROD[dc_id])
                    await self.port(443)

                await self.api_id(api_id)
                await self.test_mode(test_mode)
                await self.auth_key(auth_key)
                await self.user_id(user_id)
                await self.is_bot(is_bot)
                await self.date(0)
        else:
            await self.update()

        with self.conn:
            self.conn.execute("VACUUM")

    async def delete(self):
        try:
            os.remove(self.database)
        except FileNotFoundError:
            logger.debug(f"会话文件已不存在: {self.database}")
        except OSError as e:
            logger.warning(f"删除会话文件失败: {self.database}, 错误: {e}")
            raise


class Client(pyrogram.Client):
    def __init__(self, *args, **kw):
        super().__init__(*args, **kw)

        if self.in_memory:
            self.storage = SQLiteStorage(
                self.name, workdir=self.workdir, session_string=self.session_string, in_memory=self.in_memory
            )
        else:
            self.storage = FileStorage(
                self.name, workdir=self.workdir, session_string=self.session_string, in_memory=self.in_memory
            )

        self.dispatcher: Dispatcher = Dispatcher(self)

        self.stop_handlers = []

    async def authorize(self):
        if self.bot_token:
            return await self.sign_in_bot(self.bot_token)
        retry = False
        sent_code = await self.send_code(self.phone_number)
        code_target = {
            SentCodeType.APP: " Telegram 客户端",
            SentCodeType.SMS: "短信",
            SentCodeType.CALL: "来电",
            SentCodeType.FLASH_CALL: "闪存呼叫",
            SentCodeType.FRAGMENT_SMS: " Fragment 短信",
            SentCodeType.EMAIL_CODE: "邮件",
        }

        attempts = 0
        while True:
            try:
                if not self.phone_code:
                    if retry:
                        msg = f'验证码错误, 请重新输入 "{self.phone_number}" 的登录验证码 (按回车确认)'
                    else:
                        msg = f'请从{code_target[sent_code.type]}接收 "{self.phone_number}" 的登录验证码 (按回车确认)'
                    try:
                        self.phone_code = Prompt.ask(" " * 23 + msg, console=var.console)
                    except EOFError:
                        raise BadRequest(
                            f'登录 "{self.phone_number}" 时出现异常: 您正在使用非交互式终端, 无法输入验证码.'
                        )
                signed_in = await self.sign_in(self.phone_number, sent_code.phone_code_hash, self.phone_code)
            except (CodeInvalid, PhoneCodeInvalid):
                self.phone_code = None
                retry = True
                attempts += 1
                if attempts >= 3:
                    raise BadRequest(
                        f'登录 "{self.phone_number}" 时出现异常: 验证码尝试次数过多, 请稍后重试.'
                    )
                await asyncio.sleep(3)
            except SessionPasswordNeeded:
                retry = False
                while True:
                    if not self.password:
                        if retry:
                            msg = f'密码错误, 请重新输入 "{self.phone_number}" 的两步验证密码 (不显示, 按回车确认)'
                        else:
                            msg = f'需要输入 "{self.phone_number}" 的两步验证密码 (不显示, 按回车确认)'
                        self.password = Prompt.ask(" " * 23 + msg, password=True, console=var.console)
                    try:
                        return await self.check_password(self.password)
                    except BadRequest:
                        self.password = None
                        retry = True
            except FloodWait:
                raise BadRequest(f'登录 "{self.phone_number}" 时出现异常: 登录过于频繁.')
            except PhoneNumberInvalid:
                raise BadRequest(
                    f'登录 "{self.phone_number}" 时出现异常: 您使用了错误的手机号 (格式错误或没有注册).'
                )
            except PhoneNumberBanned:
                raise BadRequest(f'登录 "{self.phone_number}" 时出现异常: 您的账户已被封禁.')
            except Exception as e:
                logger.error(f"登录时出现异常错误!")
                show_exception(e, regular=False)
                retry = True
                attempts += 1
                if attempts >= 3:
                    raise BadRequest(f'登录 "{self.phone_number}" 时出现异常: 尝试次数过多, 请稍后重试.')
                await asyncio.sleep(3)
            else:
                break
        if isinstance(signed_in, types.User):
            return signed_in
        else:
            raise BadRequest("该账户尚未注册")

    def add_handler(self, handler: Handler, group: int = 0):
        async def dummy():
            pass

        if isinstance(handler, StartHandler):
            self.start_handler = handler.callback
            return asyncio.ensure_future(dummy())
        elif isinstance(handler, StopHandler):
            self.stop_handler = handler.callback
            return asyncio.ensure_future(dummy())
        elif isinstance(handler, ConnectHandler):
            self.connect_handler = handler.callback
            return asyncio.ensure_future(dummy())
        elif isinstance(handler, DisconnectHandler):
            self.disconnect_handler = handler.callback
            return asyncio.ensure_future(dummy())
        else:
            return self.dispatcher.add_handler(handler, group)

    def remove_handler(self, handler: Handler, group: int = 0):
        async def dummy():
            pass

        if isinstance(handler, StartHandler):
            self.start_handler = None
            return asyncio.ensure_future(dummy())
        elif isinstance(handler, StopHandler):
            self.stop_handler = None
            return asyncio.ensure_future(dummy())
        elif isinstance(handler, ConnectHandler):
            self.connect_handler = None
            return asyncio.ensure_future(dummy())
        elif isinstance(handler, DisconnectHandler):
            self.disconnect_handler = None
            return asyncio.ensure_future(dummy())
        else:
            return self.dispatcher.remove_handler(handler, group)

    @asynccontextmanager
    async def catch_reply(self, chat_id: Union[int, str], outgoing=False, filter=None):
        async def handler_func(client, message, future: asyncio.Future):
            try:
                future.set_result(message)
            except asyncio.InvalidStateError:
                pass

        future = asyncio.Future()
        f = filters.chat(chat_id)
        if not outgoing:
            f = f & (~filters.outgoing)
        if filter:
            f = f & filter
        handler = MessageHandler(async_partial(handler_func, future=future), f)
        await self.add_handler(handler, group=0)
        try:
            yield future
        finally:
            await self.remove_handler(handler, group=0)

    @asynccontextmanager
    async def catch_edit(self, message: types.Message, filter=None):
        def filter_message(id: int):
            async def func(flt, _, message: types.Message):
                return message.id == id

            return filters.create(func, "MessageFilter")

        async def handler_func(client, message, future: asyncio.Future):
            try:
                future.set_result(message)
            except asyncio.InvalidStateError:
                pass

        future = asyncio.Future()
        f = filter_message(message.id)
        if filter:
            f = f & filter
        handler = EditedMessageHandler(async_partial(handler_func, future=future), f)
        await self.add_handler(handler, group=0)
        try:
            yield future
        finally:
            await self.remove_handler(handler, group=0)

    async def wait_reply(
        self,
        chat_id: Union[int, str],
        send: str = None,
        timeout: float = 10,
        filter=None,
    ):
        async with self.catch_reply(chat_id=chat_id, filter=filter) as f:
            if send:
                await self.send_message(chat_id, send)
            msg: types.Message = await asyncio.wait_for(f, timeout)
            return msg

    async def wait_edit(
        self,
        message: types.Message,
        click: Union[str, int] = None,
        timeout: float = 10,
        noanswer=True,
        filter=None,
    ):
        async with self.catch_edit(message, filter=filter) as f:
            if click:
                try:
                    await message.click(click)
                except (TimeoutError, MessageIdInvalid):
                    if noanswer:
                        pass
                    else:
                        raise
            msg: types.Message = await asyncio.wait_for(f, timeout)
            return msg

    async def mute_chat(self, chat_id: Union[int, str], until: Union[int, datetime, None] = None):
        if until is None:
            until = 0x7FFFFFFF  # permanent mute
        elif isinstance(until, datetime):
            until = until.timestamp()
        return await self.invoke(
            raw.functions.account.UpdateNotifySettings(
                peer=raw.types.InputNotifyPeer(peer=await self.resolve_peer(chat_id)),
                settings=raw.types.InputPeerNotifySettings(
                    show_previews=False,
                    mute_until=int(until),
                ),
            )
        )

    async def handle_updates(self, updates):
        try:
            return await super().handle_updates(updates)
        except OSError as e:
            logger.warning(f"与 Telegram 服务器连接错误: {e}")
            raise
        except sqlite3.ProgrammingError:
            return



from __future__ import annotations

import asyncio
import random
import re
from typing import TYPE_CHECKING

from pyrogram.errors import MessageIdInvalid
from pyrogram.types import Message
from pyrogram.raw.types.messages import BotCallbackAnswer

from .pyrogram import Client

if TYPE_CHECKING:
    from loguru import Logger


class EmbybossRegister:
    def __init__(self, client: Client, logger: Logger, username: str, password: str):
        self.client = client
        self.log = logger
        self.username = username
        self.password = password

    async def run(self, bot: str):
        """单次注册尝试"""
        return await self._register_once(bot)

    async def run_continuous(self, bot: str, interval_seconds: int = 1):
        try:
            panel = await self.client.wait_reply(bot, "/start")
        except asyncio.TimeoutError:
            self.log.warning("初始命令无响应, 无法注册.")
            return False

        while True:
            try:
                result = await self._attempt_with_panel(panel)
                if result:
                    self.log.info(f"注册成功")
                    return True

                if interval_seconds:
                    self.log.debug(f"注册失败, {interval_seconds} 秒后重试.")
                    await asyncio.sleep(interval_seconds)
                else:
                    self.log.debug(f"注册失败, 即将重试.")
                    return False
            except (MessageIdInvalid, ValueError, AttributeError):
                # 面板失效或结构变化, 重新获取
                self.log.debug("面板失效, 正在重新获取...")
                try:
                    panel = await self.client.wait_reply(bot, "/start")
                except asyncio.TimeoutError:
                    if interval_seconds:
                        self.log.warning("重新获取面板失败, 等待后重试.")
                        await asyncio.sleep(interval_seconds)
                        continue
                    else:
                        self.log.warning("重新获取面板失败, 无法注册.")
                        return False
            except asyncio.CancelledError:
                break
            except Exception as e:
                self.log.error(f"注册异常: {e}")
                await asyncio.sleep(5)
        return False

    async def _register_once(self, bot: str):
        try:
            panel = await self.client.wait_reply(bot, "/start")
        except asyncio.TimeoutError:
            self.log.warning("初始命令无响应, 无法注册.")
            return False

        text = panel.text or panel.caption
        try:
            current_status = re.search(r"当前状态 \| ([^\n]+)", text).group(1).strip()
            register_status = re.search(r"注册状态 \| (True|False)", text).group(1) == "True"
            available_slots = int(re.search(r"可注册席位 \| (\d+)", text).group(1))
        except (AttributeError, ValueError):
            self.log.warning("无法解析界面, 无法注册, 可能您已注册.")
            return False

        if current_status != "未注册":
            self.log.warning("当前状态不是未注册, 无法注册.")
            return False
        if not register_status:
            self.log.debug(f"未开注, 将继续监控.")
            return False
        if available_slots <= 0:
            self.log.debug("可注册席位不足, 将继续监控.")
            return False

        return await self._attempt_with_panel(panel)

    async def _attempt_with_panel(self, panel: Message):
        # 点击创建账户按钮
        buttons = panel.reply_markup.inline_keyboard
        create_button = None
        for row in buttons:
            for button in row:
                if "创建账户" in button.text:
                    create_button = button.text
                    break
            if create_button:
                break

        if not create_button:
            self.log.warning("找不到创建账户按钮, 无法注册.")
            return False

        await asyncio.sleep(random.uniform(0.5, 1.5))

        async with self.client.catch_reply(panel.chat.id) as f:
            try:
                answer: BotCallbackAnswer = await panel.click(create_button)
                if "已关闭" in answer.message or answer.alert:
                    self.log.debug("未开注, 将继续监控.")
                    return False
            except (TimeoutError, MessageIdInvalid):
                pass
            try:
                msg: Message = await asyncio.wait_for(f, 5)
            except asyncio.TimeoutError:
                self.log.warning("创建账户按钮点击无响应, 无法注册.")
                return False

        text = msg.text or msg.caption
        if "您已进入注册状态" not in text:
            self.log.warning("未能正常进入注册状态, 注册失败.")
            return False

        try:
            msg = await self.client.wait_reply(msg.chat.id, f"{self.username} {self.password}")
        except asyncio.TimeoutError:
            self.log.warning("发送凭据后无响应, 无法注册.")
            return False

        msg = await self.client.wait_edit(msg)
        text = msg.text or msg.caption
        if "创建用户成功" not in text:
            self.log.warning("发送凭据后注册失败.")
            return False
        else:
            self.log.info("注册成功!")
            return True



from .pornfans_answer import _PornfansAnswerAnswerMonitor, _PornfansAnswerResultMonitor

__ignore__ = True


class TestPornfansAnswerMonitor:
    class _TestPornfansAnswerAnswerMonitor(_PornfansAnswerAnswerMonitor):
        name = "PornFans 问题回答测试"
        chat_name = "api_group"
        chat_user = ["embykeeper_test_bot"]

    class _TestPornfansAnswerResultMonitor(_PornfansAnswerResultMonitor):
        name = "PornFans 问题答案测试"
        chat_name = "api_group"
        chat_user = ["embykeeper_test_bot"]



from typing import List, Literal, Optional, Union
from loguru import logger
from pydantic import BaseModel, ValidationError

from . import Messager

__ignore__ = True


class TemplateAMessagerConfig(BaseModel):
    name: str = None
    chat_name: Optional[Union[str, int]] = None  # 发送群聊名称
    messages: List[str] = None  # 使用的语料, 与 message_lists 二选一
    message_lists: Union[str, List[str]] = (
        []
    )  # 使用的语料列表, 例如 ["some-wl@v1.yaml * 1000"], 放置在 basedir 中, 且 @v1.yaml 尾缀是必须的
    min_interval: Optional[int] = None  # 发送最小间隔 (秒)
    max_interval: Optional[int] = None  # 发送最大间隔 (秒)
    at: Optional[List[str]] = None  # 时间区间, 例如 ["5:00AM", "9:00PM"]
    possibility: Optional[float] = None  # 发送概率, 例如 1.00
    only: Optional[Literal["weekday", "weekend"]] = None  # 仅在周末/周中发送
    max_count_recent_5: int = 1
    max_count_recent_10: int = 1

    # Backward compatibility
    interval: Optional[int] = None


class TemplateAMessager(Messager):
    additional_auth = ["prime"]

    async def init(self):
        try:
            self.t_config = TemplateAMessagerConfig.model_validate(self.config)
        except ValidationError as e:
            self.log.warning(f"初始化失败: 水群器自定义模板 A 的配置错误:\n{e}")
            return False
        self.name = self.t_config.name or "自定义"
        self.chat_name = self.t_config.chat_name
        self.default_messages = self.t_config.message_lists
        self.at = self.t_config.at
        self.possibility = self.t_config.possibility
        self.only = self.t_config.only
        self.max_count_recent_5 = self.t_config.max_count_recent_5
        self.max_count_recent_10 = self.t_config.max_count_recent_10
        # messages / min_interval / max_interval 由 config 读取
        if not self.chat_name:
            self.log.warning(f"初始化失败: 没有定义任何目标群组, 请参考教程进行配置.")
            return False
        self.log = logger.bind(scheme="telemessager", name=self.name, username=self.me.full_name)
        return True


def use(**kw):
    return type("TemplatedClass", (TemplateAMessager,), kw)



from __future__ import annotations

import asyncio
from datetime import datetime
from typing import List, Dict, Type
import random
import string
import re

from loguru import logger

from embykeeper.schedule import Scheduler
from embykeeper.schema import TelegramAccount
from embykeeper.config import config
from embykeeper.runinfo import RunContext, RunStatus
from embykeeper.utils import AsyncTaskPool

from .pyrogram import Client
from .embyboss import EmbybossRegister
from .dynamic import extract, get_cls
from .link import Link
from .session import ClientsSession

logger = logger.bind(scheme="teleregistrar")


class RegisterManager:
    """注册管理器"""

    def __init__(self):
        self._tasks: Dict[str, asyncio.Task] = {}  # phone -> task
        self._schedulers: Dict[str, Scheduler] = {}  # phone -> scheduler
        self._pool = AsyncTaskPool()

        config.on_list_change("telegram.account", self._handle_account_change)
        config.on_change("registrar", self._handle_config_change)

    def _handle_config_change(self, *args):
        """Handle changes to the register configuration"""
        # Stop all existing schedulers - collect phones first
        phones = set()
        for key in self._schedulers.keys():
            if "." in key:
                phone = key.split(".")[0]
                phones.add(phone)

        for phone in phones:
            self.stop_account(phone)

        # Reschedule all accounts with the new configuration
        for account in config.telegram.account:
            if account.enabled and account.registrar:
                schedulers = self.schedule_account(account)
                if schedulers:
                    if isinstance(schedulers, list):
                        for scheduler in schedulers:
                            if hasattr(scheduler, "schedule"):
                                self._pool.add(scheduler.schedule())
                    else:
                        # 间隔注册返回的是task, 直接添加
                        self._pool.add(schedulers)

        logger.info("已根据新的配置重新安排所有注册任务.")

    def _handle_account_change(self, added: List[TelegramAccount], removed: List[TelegramAccount]):
        """Handle account additions and removals"""
        for account in removed:
            self.stop_account(account.phone)
            logger.info(f"{account.phone} 账号的注册及其计划任务已被清除.")

        for account in added:
            if account.enabled and account.registrar:
                schedulers = self.schedule_account(account)
                if schedulers:
                    if isinstance(schedulers, list):
                        for scheduler in schedulers:
                            if hasattr(scheduler, "schedule"):
                                self._pool.add(scheduler.schedule())
                    else:
                        # 间隔注册返回的是task, 直接添加
                        self._pool.add(schedulers)
                    logger.info(f"新增的 {account.phone} 账号的注册计划任务已增加.")

    def stop_account(self, phone: str):
        """Stop scheduling and running tasks for an account"""
        # Cancel all tasks for this phone
        keys_to_remove = []
        for key in self._tasks.keys():
            if key.startswith(f"{phone}."):
                self._tasks[key].cancel()
                keys_to_remove.append(key)
        for key in keys_to_remove:
            del self._tasks[key]

        # Cancel all schedulers for this phone
        keys_to_remove = []
        for key in self._schedulers.keys():
            if key.startswith(f"{phone}."):
                keys_to_remove.append(key)
        for key in keys_to_remove:
            del self._schedulers[key]

    def get_sites_for_account(self, account: TelegramAccount) -> List[str]:
        """获取账户需要注册的站点列表."""
        phone_masked = TelegramAccount.get_phone_masked(account.phone)

        sites = []
        if account.site and account.site.registrar:
            sites = account.site.registrar
        elif config.site and config.site.registrar:
            sites = config.site.registrar

        if not sites:
            logger.warning(f"{phone_masked} 账号未配置 registrar 站点, 将跳过注册调度")
            return []

        return sites

    def schedule_account(self, account: TelegramAccount) -> tuple[List[Scheduler], List[asyncio.Task]]:
        """为单个账户安排注册任务"""
        phone = account.phone
        if phone in self._schedulers or phone in self._tasks:
            self.stop_account(phone)

        # 获取此账户启用的站点
        sites_to_register_names = self.get_sites_for_account(account)
        if not sites_to_register_names:
            return [], []

        clses = extract(get_cls("registrar", names=sites_to_register_names))
        if not clses:
            logger.warning(f"{account.phone} 账号没有有效的registrar站点, 将跳过注册调度")
            return [], []

        schedulers = []
        tasks = []
        for cls in clses:
            if hasattr(cls, "templ_name"):
                site_name = cls.templ_name  # "templ_a<XiguaEmbyBot>"
            else:
                site_name = cls.__module__.rsplit(".", 1)[-1]

            site_config = config.registrar.get_site_config(site_name)
            if not site_config:
                logger.warning(f"{account.phone} 账号的站点 {site_name} 未配置注册设置, 将跳过")
                continue

            if site_config.get("times"):
                # 定时模式
                scheduler = self._schedule_site_timed(account, site_name, site_config)
                schedulers.append(scheduler)
            elif site_config.get("interval_minutes"):
                # 间隔模式
                task = self._schedule_site_interval(account, site_name, site_config)
                tasks.append(task)

        return schedulers, tasks

    def _schedule_site_timed(self, account: TelegramAccount, site_name: str, site_config: dict):
        """定时注册模式"""
        phone_masked = TelegramAccount.get_phone_masked(account.phone)
        times = site_config.get("times", [])

        # 将时间列表转换为时间范围格式
        times_str = ",".join(times)
        time_range = f"<{times_str}>"

        def on_next_time(t: datetime):
            logger.info(
                f"下一次 \"{phone_masked}\" 账号 {site_name} 站点的注册将在 {t.strftime('%m-%d %H:%M %p')} 进行."
            )
            date_ctx = RunContext.get_or_create(f"registrar.date.{t.strftime('%Y%m%d')}")
            account_ctx = RunContext.get_or_create(f"registrar.account.{account.phone}")
            site_ctx = RunContext.get_or_create(f"registrar.site.{site_name}")
            return RunContext.prepare(
                description=f"{account.phone} 账号 {site_name} 站点定时注册",
                parent_ids=[account_ctx.id, date_ctx.id, site_ctx.id],
            )

        def func(ctx: RunContext):
            task = asyncio.create_task(self._run_single_site(ctx, account, site_name, site_config))
            log = logger.bind(username=f"@{site_name}", name=f"{phone_masked}")
            log.info(f"已计划定时抢注任务, 下次运行: {scheduler.next_run.strftime('%Y-%m-%d %H:%M:%S')}")
            return task

        scheduler = Scheduler.from_str(
            func=func,
            interval_days="1",  # 每天执行
            time_range=time_range,
            on_next_time=on_next_time,
            description=f"{account.phone} 账号 {site_name} 站点定时注册任务",
            sid=f"registrar.timed.{account.phone}.{site_name}",
        )

        scheduler_key = f"{account.phone}.{site_name}"
        self._schedulers[scheduler_key] = scheduler
        return scheduler

    def _schedule_site_interval(self, account: TelegramAccount, site_name: str, site_config: dict):
        """间隔注册模式"""
        interval_minutes = site_config.get("interval_minutes")
        task_key = f"{account.phone}.{site_name}"

        if interval_minutes < 3:
            # 连续模式
            task = asyncio.create_task(self._continuous_register_task(account, site_name, site_config))
        else:
            # 间隔模式
            task = asyncio.create_task(
                self._interval_register_task(account, site_name, site_config, interval_minutes)
            )

        self._tasks[task_key] = task
        return task

    async def _continuous_register_task(self, account: TelegramAccount, site_name: str, site_config: dict):
        """连续注册任务"""
        interval_minutes = site_config.get("interval_minutes")

        async with ClientsSession([account]) as clients:
            async for a, client in clients:
                match = re.match(r"templ_a<(.+?)>", site_name)
                if not match:
                    logger.error(f"无法从 {site_name} 中提取机器人用户名")
                    return
                bot_username = match.group(1)

                log = logger.bind(name=f"{client.me.full_name}, @{bot_username}")
                log.info(f"开始连续注册, 间隔 {interval_minutes} 分钟.")

                if not await Link(client).auth("registrar", log_func=log.error):
                    log.error("账户权限验证失败.")
                    return

                embyboss_register = EmbybossRegister(
                    client=client,
                    logger=log,
                    username=client.me.username or f"user_{client.me.id}",
                    password="".join(random.choices(string.ascii_letters + string.digits, k=4)),
                )

                async def long_running_task():
                    try:
                        await embyboss_register.run_continuous(bot_username, interval_minutes * 60)
                    except asyncio.CancelledError:
                        log.info("连续注册任务被取消.")
                    except Exception as e:
                        log.error(f"连续注册任务出现异常: {e}")
                        logger.exception("详细异常信息:")

                task = asyncio.create_task(long_running_task())
                client.stop_handlers.append(task.cancel)
                try:
                    await task
                finally:
                    if task.cancel in client.stop_handlers:
                        client.stop_handlers.remove(task.cancel)

    async def _interval_register_task(
        self, account: TelegramAccount, site_name: str, site_config: dict, interval_minutes: int
    ):
        """间隔注册任务"""

        async with ClientsSession([account]) as clients:
            async for a, client in clients:
                match = re.match(r"templ_a<(.+?)>", site_name)
                bot_username = match.group(1) if match else site_name

                log = logger.bind(name=f"{client.me.full_name}, @{bot_username}")

                async def loop():
                    while True:
                        try:
                            account_ctx = RunContext.get_or_create(f"registrar.account.{account.phone}")
                            await RunContext.run(
                                lambda c: self._run_single_site(c, account, site_name, site_config),
                                description=f"{account.phone} 账号 {site_name} 站点间隔注册",
                                parent_ids=[account_ctx.id],
                            )
                        except Exception as e:
                            log.error(f"间隔注册任务异常: {e}")
                            logger.exception("详细异常信息:")

                        await asyncio.sleep(interval_minutes * 60)

                task = asyncio.create_task(loop())
                client.stop_handlers.append(task.cancel)
                try:
                    await task
                finally:
                    if task.cancel in client.stop_handlers:
                        client.stop_handlers.remove(task.cancel)

    async def _run_single_site(
        self, ctx: RunContext, account: TelegramAccount, site_name: str, site_config: dict
    ):
        """运行单个站点的注册任务"""

        async with ClientsSession([account]) as clients:
            async for a, client in clients:
                match = re.match(r"templ_a<(.+?)>", site_name)
                bot_username = match.group(1) if match else site_name

                log = logger.bind(name=f"{client.me.full_name}, @{bot_username}")

                if not await Link(client).auth("registrar", log_func=log.error):
                    log.error("账户权限验证失败.")
                    return

                clses = extract(get_cls("registrar", names=[site_name]))
                if not clses:
                    log.error(f"无法找到站点 {site_name} 的注册器")
                    return

                cls = clses[0]
                registrar = cls(client=client, **site_config)
                await registrar.start()

    async def schedule_all(self) -> tuple[List[Scheduler], List[asyncio.Task]]:
        """安排所有注册任务."""
        logger.debug(f"开始为所有账户安排注册任务, 总账户数: {len(config.telegram.account)}")

        all_schedulers = []
        all_tasks = []

        for a in config.telegram.account:
            logger.debug(
                f"检查账户 {a.phone}: enabled={a.enabled}, registrar={getattr(a, 'registrar', False)}"
            )
            if a.enabled and getattr(a, "registrar", False):
                logger.debug(f"为账户 {a.phone} 安排注册任务")
                schedulers_for_account, tasks_for_account = self.schedule_account(a)
                all_schedulers.extend(schedulers_for_account)
                all_tasks.extend(tasks_for_account)

        logger.debug(f"最终调度器数量: {len(all_schedulers)}, 任务数量: {len(all_tasks)}")
        return all_schedulers, all_tasks

    async def start(self):
        """安排所有注册任务并等待常驻任务完成."""
        schedulers, tasks = await self.schedule_all()

        if not schedulers and not tasks:
            logger.info("没有需要执行的 Telegram 机器人注册任务")
            return

        logger.info(f"已创建 {len(schedulers)} 个定时注册调度器和 {len(tasks)} 个间隔注册任务.")

        awaitables = tasks + [s.schedule() for s in schedulers]

        if awaitables:
            await asyncio.gather(*awaitables)

    async def _interval_register_task(
        self, account: TelegramAccount, site_name: str, site_config: dict, interval_minutes: int
    ):
        phone_masked = TelegramAccount.get_phone_masked(account.phone)

        async with ClientsSession([account]) as clients:
            async for _, client in clients:
                match = re.match(r"templ_a<(.+?)>", site_name)
                bot_username = match.group(1) if match else site_name

                log = logger.bind(name=f"{client.me.full_name}, @{bot_username}")

                async def loop():
                    while True:
                        try:
                            account_ctx = RunContext.get_or_create(f"register.account.{account.phone}")
                            site_ctx = RunContext.get_or_create(f"register.site.{site_name}")
                            ctx = RunContext.prepare(
                                description=f"{client.me.full_name} 账号 {site_name} 站点间隔注册",
                                parent_ids=[account_ctx.id, site_ctx.id],
                            )

                            await self._run_single_site(ctx, account, site_name, site_config)
                            await asyncio.sleep(interval_minutes * 60)

                        except asyncio.CancelledError:
                            break
                        except Exception as e:
                            logger.error(f"{phone_masked} 账号 {site_name} 站点注册异常: {e}")
                            await asyncio.sleep(interval_minutes * 60)

                task = asyncio.create_task(loop())
                client.stop_handlers.append(task.cancel)
                try:
                    await task
                finally:
                    if task.cancel in client.stop_handlers:
                        client.stop_handlers.remove(task.cancel)

    async def run_account(self, ctx: RunContext, account: TelegramAccount, instant: bool = False):
        """Run register for a single account"""
        async with ClientsSession([account]) as clients:
            async for a, client in clients:
                await self._run_account(ctx, a, client, instant)

    async def _run_single_site(
        self, ctx: RunContext, account: TelegramAccount, site_name: str, site_config: dict
    ):
        """运行单个站点的注册"""
        async with ClientsSession([account]) as clients:
            async for _, client in clients:
                match = re.match(r"templ_a<(.+?)>", site_name)
                bot_username = match.group(1) if match else site_name

                log = logger.bind(name=f"{client.me.full_name}, @{bot_username}")

                if not await Link(client).auth("registrar", log_func=log.error):
                    return

                cls = get_cls("registrar", names=[site_name])[0]

                register = cls(
                    client,
                    context=ctx,
                    retries=site_config.get("retries", 1),
                    timeout=site_config.get("timeout", 120),
                    config=site_config,
                )

                result = await register._start()
                if result.status == RunStatus.SUCCESS:
                    logger.bind(username=f"@{site_name}", name=f"{client.me.full_name}").info("注册成功")
                elif result.status == RunStatus.IGNORE:
                    logger.bind(username=f"@{site_name}", name=f"{client.me.full_name}").info("跳过注册")
                else:
                    logger.bind(username=f"@{site_name}", name=f"{client.me.full_name}").warning("注册失败")

    async def _run_account(
        self, ctx: RunContext, account: TelegramAccount, client: Client, instant: bool = False
    ):
        """Run registers for a single user"""
        log = logger.bind(username=client.me.full_name)

        # Get register classes based on account config or global config
        site = None
        if account.site and account.site.registrar is not None:
            site = account.site.registrar
        elif config.site and config.site.registrar is not None:
            site = config.site.registrar
        else:
            log.warning("没有配置registrar站点, 注册将跳过.")
            return

        clses: List[Type] = extract(get_cls("registrar", names=site))

        if not clses:
            log.warning("没有任何有效注册站点, 注册将跳过.")
            return

        if not await Link(client).auth("registrar", log_func=log.error):
            return

        config_to_use = account.registrar_config or config.registrar
        sem = asyncio.Semaphore(config_to_use.concurrency)
        registers = []

        for cls in clses:
            if hasattr(cls, "templ_name"):
                site_name = cls.templ_name
            else:
                site_name = cls.__module__.rsplit(".", 1)[-1]

            site_config = config_to_use.get_site_config(site_name)
            if not site_config:
                log.warning(f"站点 {site_name} 未配置注册设置, 将跳过")
                continue

            site_ctx = RunContext.prepare(f"{site_name} 站点注册", parent_ids=ctx.id)
            registers.append(
                cls(
                    client,
                    context=site_ctx,
                    retries=site_config.get("retries", 1),
                    timeout=site_config.get("timeout", 120),
                    config=site_config,
                )
            )

        if not registers:
            log.warning("所有站点都未正确配置, 注册将跳过.")
            return

        tasks = []
        names = []
        for r in registers:
            names.append(f"@{r.bot_username}" if hasattr(r, "bot_username") and r.bot_username else r.name)
            task = self._task_main(r, sem)
            tasks.append(task)

        if names:
            logger.info(f'已启用注册器: {", ".join(names)}')

        results = await asyncio.gather(*tasks)

        failed = []
        successful = []
        ignored = []

        for r, result in results:
            if result.status == RunStatus.SUCCESS:
                successful.append(r.name)
            elif result.status == RunStatus.IGNORE:
                ignored.append(r.name)
            else:
                failed.append(r.name)

        spec = f"共{len(successful) + len(failed) + len(ignored)}个"
        if successful:
            spec += f", {len(successful)}成功"
        if failed:
            spec += f", {len(failed)}失败"
        if ignored:
            spec += f", {len(ignored)}跳过"

        if failed:
            msg = "注册部分失败" if successful else "注册失败"
            logger.bind(username=client.me.full_name).error(f"{msg} ({spec}): {', '.join(failed)}")
        else:
            logger.bind(username=client.me.full_name).info(f"注册完成 ({spec}).")

    async def _task_main(self, register, sem: asyncio.Semaphore):
        # 注册器不需要等待, 立即执行
        async with sem:
            result = await register._start()
            return register, result

    async def run_all(self, instant: bool = False):
        """Run registers for all enabled accounts without scheduling"""
        accounts = [a for a in config.telegram.account if a.enabled and getattr(a, "registrar", False)]
        tasks = [
            asyncio.create_task(self.run_account(RunContext.prepare("运行全部注册器"), account, instant))
            for account in accounts
        ]
        try:
            await asyncio.gather(*tasks)
        except asyncio.CancelledError:
            for task in tasks:
                if not task.done():
                    task.cancel()
            await asyncio.gather(*tasks, return_exceptions=True)
            raise

    async def run_single_bot(self, bot_username: str, instant: bool = True):
        """快速注册单个bot - 用于-R命令"""
        accounts = [a for a in config.telegram.account if a.enabled]

        if not accounts:
            logger.error("没有可用的Telegram账号")
            return

        # 为每个账号创建快速注册任务
        tasks = []
        for account in accounts:
            task = asyncio.create_task(self._run_single_bot_for_account(account, bot_username))
            tasks.append(task)

        try:
            await asyncio.gather(*tasks)
        except asyncio.CancelledError:
            for task in tasks:
                if not task.done():
                    task.cancel()
            await asyncio.gather(*tasks, return_exceptions=True)
            raise

    async def _run_single_bot_for_account(self, account: TelegramAccount, bot_username: str):
        async with ClientsSession([account]) as clients:
            async for a, client in clients:
                log = logger.bind(name=f"{client.me.full_name}, @{bot_username}")

                if not await Link(client).auth("registrar", log_func=log.error):
                    return

                embyboss_register = EmbybossRegister(
                    client=client,
                    logger=log,
                    username=client.me.username or f"user_{client.me.id}",
                    password="".join(random.choices(string.ascii_letters + string.digits, k=4)),
                )

                task = asyncio.create_task(embyboss_register.run_continuous(bot_username, 1))
                client.stop_handlers.append(task.cancel)
                try:
                    await task
                finally:
                    if task.cancel in client.stop_handlers:
                        client.stop_handlers.remove(task.cancel)
                logger.bind(name=f"{client.me.full_name}, @{bot_username}").info("快速抢注任务已完成.")



from .pornfans_exam import _PornfansExamAnswerMonitor

__ignore__ = True


class TestPornfansExamMonitor(_PornfansExamAnswerMonitor):
    name = "PornFans 科举回答测试"
    chat_name = "api_group"
    chat_user = ["embykeeper_test_bot"]



import apprise
from loguru import logger
from rich.text import Text

logger = logger.bind(scheme="notifier", nonotify=True)


class AppriseStream:
    def __init__(self, uri: str):
        self.apobj = apprise.Apprise()
        self.apobj.add(uri)

    def write(self, message):
        # The message from loguru has a newline at the end, remove it.
        message = message.strip()
        # The message is formatted as "LEVEL#MESSAGE"
        level, _, body = message.partition("#")
        level = level.lower()
        body = Text.from_markup(body).plain

        # Map loguru levels to apprise levels
        notify_type = apprise.NotifyType.INFO
        if level == "warning":
            notify_type = apprise.NotifyType.WARNING
        elif level == "error" or level == "critical":
            notify_type = apprise.NotifyType.FAILURE
        elif level == "success":
            notify_type = apprise.NotifyType.SUCCESS

        if not self.apobj.notify(body=body, title="Embykeeper", notify_type=notify_type):
            logger.warning(f"Failed to send notification via Apprise.")

    def close(self):
        pass

    async def join(self):
        pass



import logging
import os
from pathlib import Path
import sys
from typing import List, Optional
from functools import wraps

import typer
import asyncio
from loguru import logger
from appdirs import user_data_dir

from . import var, __author__, __name__ as __product__, __url__, __version__
from .utils import AsyncTaskPool, show_exception
from .config import config


class AsyncTyper(typer.Typer):
    def async_command(self, *args, **kwargs):
        def decorator(async_func):
            @wraps(async_func)
            def sync_func(*_args, **_kwargs):
                async def main():
                    try:
                        await async_func(*_args, **_kwargs)
                    except typer.Exit as e:
                        return e.exit_code
                    except Exception as e:
                        print("\r", end="", flush=True)
                        logger.critical(f"发生关键错误, {__product__.capitalize()} 将退出.")
                        show_exception(e, regular=False)
                        return 1
                    else:
                        logger.info(f"所有任务已完成, 欢迎您再次使用 {__product__.capitalize()}.")

                returncode = 130
                try:
                    loop = asyncio.new_event_loop()
                    asyncio.set_event_loop(loop)
                    returncode = loop.run_until_complete(main())
                except KeyboardInterrupt:
                    print("\r正在停止...\r", end="", flush=True, file=sys.stderr)
                finally:
                    if var.exit_handlers:
                        logger.debug("开始执行退出处理程序.")
                        try:
                            # Wait for exit handlers with timeout
                            loop.run_until_complete(
                                asyncio.wait_for(
                                    asyncio.gather(*[h() for h in var.exit_handlers], return_exceptions=True),
                                    timeout=3,
                                )
                            )
                        except asyncio.TimeoutError:
                            logger.warning("部分退出处理程序超时未完成.")
                        else:
                            logger.debug("退出处理程序执行完成, 开始清理所有任务.")
                    else:
                        logger.debug("未注册退出处理程序, 开始清理所有任务.")

                    # Then cancel remaining tasks
                    tasks = asyncio.all_tasks(loop)
                    for task in tasks:
                        task.cancel()
                    loop.run_until_complete(asyncio.gather(*tasks, return_exceptions=True))
                    loop.run_until_complete(loop.shutdown_asyncgens())
                    print("\r", end="", flush=True)
                    logger.info(f"所有服务已停止并登出, 欢迎您再次使用 {__product__.capitalize()}.")
                    raise typer.Exit(returncode)

            self.command(*args, **kwargs)(sync_func)
            return async_func

        return decorator


app = AsyncTyper(
    pretty_exceptions_enable=False,
    rich_markup_mode="rich",
    add_completion=False,
    add_help_option=False,
)


def version(flag):
    if flag:
        print(__version__)
        raise typer.Exit()


def print_example_config(flag):
    if flag:
        print(config.generate_example_config())
        raise typer.Exit()


def print_help(ctx: typer.Context, param: typer.CallbackParam, value: bool):
    if not value or ctx.resilient_parsing:
        return
    typer.echo(ctx.get_help())
    raise typer.Exit()


@app.async_command(
    help=(
        f"欢迎使用 [orange3]{__product__.capitalize()}[/] {__version__} " ":cinema: 无参数默认开启全部功能."
    )
)
async def main(
    config_file: Path = typer.Argument(
        None,
        dir_okay=False,
        allow_dash=True,
        envvar=f"EK_CONFIG_FILE",
        rich_help_panel="参数",
        help="配置文件 (置空以生成)",
    ),
    help: bool = typer.Option(
        None,
        "--help",
        "-h",
        callback=print_help,
        is_eager=True,
        rich_help_panel="调试参数",
        help="显示此帮助信息并退出.",
    ),
    checkiner: bool = typer.Option(
        False,
        "--checkin",
        "-c",
        rich_help_panel="模块开关",
        help="仅启用 Telegram 签到功能",
    ),
    emby: bool = typer.Option(
        False,
        "--emby",
        "-e",
        rich_help_panel="模块开关",
        help="仅启用 Emby 保活功能",
    ),
    subsonic: bool = typer.Option(
        False,
        "--subsonic",
        "-S",
        rich_help_panel="模块开关",
        help="仅启用 Subsonic 保活功能",
    ),
    monitor: bool = typer.Option(
        False,
        "--monitor",
        "-m",
        rich_help_panel="模块开关",
        help="仅启用群聊监视功能",
    ),
    messager: bool = typer.Option(
        False,
        "--messager",
        "-s",
        rich_help_panel="模块开关",
        help="仅启用自动水群功能",
    ),
    registrar: bool = typer.Option(
        False,
        "--registrar",
        "-r",
        rich_help_panel="模块开关",
        help="仅启用注册功能",
    ),
    registrar_bot: Optional[str] = typer.Option(
        None,
        "--registrar-bot",
        "-R",
        rich_help_panel="模块开关",
        help="快速反复尝试注册指定机器人 (Embyboss)",
    ),
    version: bool = typer.Option(
        None,
        "--version",
        "-v",
        rich_help_panel="调试参数",
        callback=version,
        is_eager=True,
        help=f"打印 {__product__.capitalize()} 版本",
    ),
    example_config: bool = typer.Option(
        None,
        "--example-config",
        "-E",
        hidden=True,
        callback=print_example_config,
        is_eager=True,
        help=f"输出范例配置文件",
    ),
    instant: bool = typer.Option(
        False,
        "--instant/--no-instant",
        "-i/-I",
        envvar="EK_INSTANT",
        show_envvar=False,
        rich_help_panel="调试参数",
        help="启动时立刻执行一次任务",
    ),
    once: bool = typer.Option(
        False,
        "--once/--cron",
        "-o/-O",
        rich_help_panel="调试参数",
        help="只执行一次而不进入计划执行模式",
    ),
    verbosity: int = typer.Option(
        False,
        "--debug",
        "-d",
        count=True,
        envvar="EK_DEBUG",
        show_envvar=False,
        rich_help_panel="调试参数",
        help="开启调试模式",
    ),
    debug_cron: bool = typer.Option(
        False,
        "--debug-cron",
        envvar="EK_DEBUG_CRON",
        show_envvar=False,
        rich_help_panel="调试工具",
        help="开启任务调试模式, 在三秒后立刻开始执行计划任务",
    ),
    debug_notify: bool = typer.Option(
        False,
        "--debug-notify",
        show_envvar=False,
        rich_help_panel="调试工具",
        help="开启日志调试模式, 发送一条日志记录和即时日志记录后退出",
    ),
    simple_log: bool = typer.Option(
        False,
        "--simple-log",
        "-L",
        rich_help_panel="调试参数",
        help="简化日志输出格式",
    ),
    disable_color: bool = typer.Option(
        False,
        "--disable-color",
        "-C",
        rich_help_panel="调试参数",
        help="禁用日志颜色",
    ),
    follow: bool = typer.Option(
        False,
        "--follow",
        "-F",
        rich_help_panel="调试工具",
        help="仅启动消息调试",
    ),
    analyze: bool = typer.Option(
        False,
        "--analyze",
        "-A",
        rich_help_panel="调试工具",
        help="仅启动历史信息分析",
    ),
    dump: List[str] = typer.Option(
        [],
        "--dump",
        "-D",
        rich_help_panel="调试工具",
        help="仅启动更新日志",
    ),
    top: bool = typer.Option(
        False,
        "--top",
        "-T",
        rich_help_panel="调试参数",
        help="执行过程中显示系统状态底栏",
    ),
    play: str = typer.Option(
        None,
        "--play-url",
        "-p",
        rich_help_panel="调试工具",
        help="仅模拟观看一个视频",
    ),
    save: bool = typer.Option(
        False,
        "--save",
        rich_help_panel="调试参数",
        help="记录执行过程中的原始更新日志",
    ),
    telegram_test_server: bool = typer.Option(
        False,
        "--telegram-test-server",
        rich_help_panel="调试参数",
        hidden=True,
        help="使用 Telegram 测试服务器",
    ),
    public: bool = typer.Option(
        False,
        "--public",
        "-P",
        hidden=True,
        rich_help_panel="调试参数",
        help="启用公共仓库部署模式",
    ),
    windows: bool = typer.Option(
        False,
        "--windows",
        "-W",
        hidden=True,
        rich_help_panel="调试参数",
        help="启用 Windows 安装部署模式",
    ),
    basedir: Path = typer.Option(
        None,
        "--basedir",
        "-B",
        rich_help_panel="调试参数",
        help="设定账号文件的位置",
    ),
    noexit: bool = typer.Option(
        False,
        "--noexit",
        "-N",
        rich_help_panel="调试参数",
        help="要求所有长期任务在没有账号时继续监控等待",
    ),
    clean: bool = typer.Option(
        False,
        "--clean",
        rich_help_panel="调试工具",
        help="显示或清理 Emby 模拟设备和登陆凭据等缓存",
    ),
):
    from .log import initialize, apply_logging_adapter

    var.debug = verbosity
    if verbosity >= 3:
        level = 0
        if verbosity < 4:
            logging.getLogger("pyrogram.session").setLevel(20)
        logging.getLogger("hpack").setLevel(20)
        asyncio.get_event_loop().set_debug(True)
        apply_logging_adapter(level=10)
    elif verbosity >= 1:
        level = "DEBUG"
    else:
        level = "INFO"

    initialize(level=level, show_path=verbosity and (not simple_log), show_time=not simple_log)
    if disable_color:
        var.console.no_color = True

    msg = " 您可以通过 Ctrl+C 以结束运行." if not public else ""
    logger.info(f"欢迎使用 [orange3]{__product__.capitalize()}[/]! 正在启动, 请稍等.{msg}")
    logger.info(f"当前版本 ({__version__}) 项目页: {__url__}")
    logger.debug(f'命令行参数: "{" ".join(sys.argv[1:])}".')

    basedir = Path(basedir or user_data_dir(__product__))
    basedir.mkdir(parents=True, exist_ok=True)
    if public:
        logger.info(f'工作目录: "{basedir}"')
    else:
        logger.info(f'工作目录: "{basedir}", 您的用户数据相关文件将存储在此处, 请妥善保管.')
        docker = bool(os.environ.get("EK_IN_DOCKER", False))
        if docker:
            logger.info("当前在 Docker 容器中运行, 请确认该目录已挂载, 否则文件将在容器重建后丢失.")
    if verbosity:
        logger.warning(f"您当前处于调试模式: 日志等级 {verbosity}.")
        app.pretty_exceptions_enable = True
    var.telegram_test_server = telegram_test_server
    if telegram_test_server:
        logger.warning("您当前处于 Telegram 测试服务器模式, 请谨慎使用.")

    config.basedir = basedir
    config.windows = windows
    config.public = public

    if public:
        from .public import public_preparation

        if not await public_preparation():
            raise typer.Exit(1)
    else:
        if not await config.reload_conf(config_file):
            raise typer.Exit(1)

    if verbosity >= 2:
        config.nofail = False
    if not config.nofail:
        logger.warning(f"您当前处于调试模式: 错误将会导致程序停止运行.")
    if debug_cron:
        config.debug_cron = True
        logger.warning("您当前处于计划任务调试模式, 将在 10 秒后运行计划任务.")
    config.noexit = noexit

    if not checkiner and not monitor and not emby and not messager and not subsonic and not registrar:
        checkiner = True
        emby = True
        subsonic = True
        monitor = True
        messager = True
        registrar = True

    config.on_change(
        "proxy", lambda x, y: logger.bind(scheme="config").warning("修改代理设置后, 可能需要重启程序以生效.")
    )

    if config.mongodb and not var.use_mongodb_config:
        if config.proxy:
            logger.warning("由于不支持, 不使用设定的代理连接 MongoDB 服务器.")
        if not public:
            logger.warning("在本地部署模式下, 不推荐设定使用 MongoDB 缓存.")
        logger.info(f"正在连接到 MongoDB 缓存, 请稍候.")
        try:
            from .cache import cache

            cache.set("test", "test")
            assert cache.get("test", None) == "test"
            cache.delete("test")
        except Exception as e:
            logger.error(f"MongoDB 缓存连接失败: {e}, 程序将退出.")
            show_exception(e, regular=False)
            return
    else:
        try:
            from .cache import cache

            cache.set("test", "test")
            assert cache.get("test", None) == "test"
            cache.delete("test")
        except Exception as e:
            logger.error(f"本地缓存读写失败: {e}, 请使用 MongoDB 缓存, 程序将退出.")
            show_exception(e, regular=False)
            return

    if clean:
        from .clean import cleaner

        return await cleaner()

    if follow:
        from .telegram.debug import follower

        return await follower()

    if top:
        from .topper import topper

        if not (var.console.is_terminal and var.console.is_interactive):
            logger.warning("在非交互模式下启用底栏可能会导致显示异常.")
        asyncio.create_task(topper())

    if play:
        from .emby.main import EmbyManager

        return await EmbyManager().play_url(play)

    if save:
        from .telegram.debug import saver

        asyncio.create_task(saver())

    if analyze:
        from .telegram.debug import analyzer

        indent = " " * 23
        chats = typer.prompt(indent + "请输入群组用户名 (以空格分隔)").split()
        keywords = typer.prompt(indent + "请输入关键词 (以空格分隔)", default="", show_default=False)
        keywords = keywords.split() if keywords else []
        timerange = typer.prompt(indent + '请输入时间范围 (以"-"分割)', default="", show_default=False)
        timerange = timerange.split("-") if timerange else []
        limit = typer.prompt(indent + "请输入各群组最大获取数量", default=10000, type=int)
        outputs = typer.prompt(indent + "请输入最大输出数量", default=1000, type=int)
        return await analyzer(chats, keywords, timerange, limit, outputs)

    if dump:
        from .telegram.debug import dumper

        return await dumper(dump)

    if debug_notify:
        from .notify import debug_notifier

        return await debug_notifier()

    try:
        checkin_man = None
        if checkiner:
            from .telegram.checkin_main import CheckinerManager

            checkin_man = CheckinerManager()

        monitor_man = None
        if monitor:
            from .telegram.monitor_main import MonitorManager

            monitor_man = MonitorManager()

        message_man = None
        if messager:
            from .telegram.message_main import MessageManager

            message_man = MessageManager()

        register_man = None
        if registrar or registrar_bot:
            from .telegram.registrar_main import RegisterManager

            register_man = RegisterManager()

        emby_man = None
        if emby:
            from .emby.main import EmbyManager

            emby_man = EmbyManager()

        subsonic_man = None
        if subsonic:
            from .subsonic.main import SubsonicManager

            subsonic_man = SubsonicManager()

        pool = AsyncTaskPool()

        if registrar_bot:
            logger.info(f"开始快速注册 @{registrar_bot}")
            if register_man:
                await register_man.run_single_bot(registrar_bot, instant=True)
            else:
                logger.error("注册管理器未初始化")
            return

        if instant and not debug_cron:
            if checkin_man:
                pool.add(checkin_man.run_all(instant=True), "站点签到")
            if emby_man:
                pool.add(emby_man.run_all(instant=True), "Emby 保活")
            if subsonic_man:
                pool.add(subsonic_man.run_all(instant=True), "Subsonic 保活")
            await pool.wait()
            logger.debug("启动时立刻执行签到和保活: 已完成.")
        if (not once) or config.noexit:
            from .notify import start_notifier

            streams = await start_notifier()
        if not once:
            if checkin_man:
                pool.add(checkin_man.schedule_all(), "站点签到")
            if register_man:
                pool.add(register_man.start(), "站点注册")
            if monitor_man:
                pool.add(monitor_man.run_all(), "群组监控")
            if message_man:
                pool.add(message_man.run_all(), "自动水群")
            if emby_man:
                pool.add(emby_man.schedule_all(), "Emby 保活")
            if subsonic_man:
                pool.add(subsonic_man.schedule_all(), "Subsonic 保活")
        if config.noexit:
            logger.info("处于长期监控模式, 当没有账号时将继续监控等待新配置.")
            pool.add(asyncio.Event().wait(), "账号配置文件监控")
        try:
            async for t in pool.as_completed():
                try:
                    await t
                except asyncio.CancelledError:
                    logger.debug(f"任务 {t.get_name()} 被取消.")
                except Exception as e:
                    logger.debug(f"任务 {t.get_name()} 出现错误, 模块可能停止运行.")
                    show_exception(e, regular=False)
                    if not config.nofail:
                        raise
                else:
                    logger.debug(f"任务 {t.get_name()} 成功结束.")
        finally:
            if streams:
                await asyncio.gather(*[stream.join() for stream in streams])
    finally:
        from .runinfo import RunContext

        RunContext.cancel_all()


if __name__ == "__main__":
    app()



import asyncio
from contextlib import asynccontextmanager
from datetime import datetime, time
import random

from pyrogram import filters
from pyrogram.types import Message
from pyrogram.enums import MessageEntityType

from embykeeper.runinfo import RunStatus

from ..messager._smart import SmartMessager
from ..lock import pornfans_alert, pornfans_messager_mids_lock, pornfans_messager_mids
from . import BotCheckin

__ignore__ = True


class SmartPornfansCheckinMessager(SmartMessager):
    name = "PornFans 签到群发言"
    chat_name = "PornFans_Chat"
    default_messages = "pornemby-checkin-wl@latest.yaml"
    additional_auth = ["pornemby_pack"]
    msg_per_day = 1
    force_day = True
    at = [time(6, 0), time(23, 59)]
    extra_prompt = "输出内容必须大于 8 个字符, 包括符号"

    async def init(self):
        async with pornfans_messager_mids_lock:
            if self.me.id not in pornfans_messager_mids:
                pornfans_messager_mids[self.me.id] = []
        return True

    async def send(self, dummy=False):
        if pornfans_alert.get(self.me.id, False):
            self.log.info(f"由于风险急停取消发送.")
            return
        message = await super().send(dummy=dummy)
        if message:
            pornfans_messager_mids[self.me.id].append(message)
        return message


class PornfansGroupCheckin(BotCheckin):
    name = "PornFans 主群发言"
    bot_username = "Porn_Emby_Bot"
    chat_name = "PornFans_Chat"
    additional_auth = ["pornemby_pack"]
    bot_use_captcha = False

    @asynccontextmanager
    async def listener(self):
        yield

    async def send_checkin(self):
        async def mention_user_filter(flt, __, m: Message):
            if m.entities:
                for e in m.entities:
                    if e.type == MessageEntityType.TEXT_MENTION:
                        if e.user.id == flt.user_id:
                            return True
            return False

        mention = filters.create(mention_user_filter, user_id=self.client.me.id)

        today_start = datetime.combine(datetime.now().date(), time(0, 0))
        async for m in self.client.search_messages(
            self.chat_name, "签到成功", max_date=today_start, from_user=self.bot_username
        ):
            if await mention(self.client, m):
                self.log.info(f"今日已经签到过了.")
                return await self.finish(RunStatus.NONEED, "今日已签到")

        messager = SmartPornfansCheckinMessager(
            self.client, config={"extra_prompt": "请注意: 回复中必须含有签到两个字, 且长度大于8个字!"}
        )

        for _ in range(10):
            async with self.client.catch_reply(
                self.chat_name, filter=mention & filters.user(self.bot_username)
            ) as f:
                msg = await messager.send()
                if not msg:
                    self.log.info(f"发送失败, 正在重试.")
                    continue
                try:
                    r_msg: Message = await asyncio.wait_for(f, 10)
                except asyncio.TimeoutError:
                    wait = random.uniform(180, 360)
                    self.log.info(f"机器人没有回应, 尝试在 {wait:.0f} 秒后重新签到")
                    await asyncio.sleep(wait)
                    continue
                else:
                    if r_msg.text and "签到成功" in r_msg.text:
                        self.log.info("[yellow]签到成功[/]")
                        return await self.finish()
                finally:
                    try:
                        await msg.delete()
                    except:
                        pass
        else:
            self.log.warning(f"签到失败: 重试超限.")
            return await self.fail()



import asyncio
import random
from pyrogram.types import Message
from pyrogram.errors import MessageIdInvalid

from ._templ_a import TemplateACheckin


class MeowCheckin(TemplateACheckin):
    name = "飞了个喵"
    bot_username = "gymeowfly_bot"

    async def message_handler(self, client, message: Message):
        if message.caption and "请先验证你不是机器人" in message.caption and message.reply_markup:
            keys = [k.text for r in message.reply_markup.inline_keyboard for k in r]
            for k in keys:
                if "我不是机器人" in k:
                    await asyncio.sleep(random.uniform(0.5, 1.5))
                    try:
                        await message.click(k)
                    except TimeoutError:
                        self.log.debug(f"点击签到按钮无响应, 可能按钮未正确处理点击回复. 一般来说不影响签到.")
                    except MessageIdInvalid:
                        pass
                    return
            else:
                self.log.warning(f"签到失败: 账户错误.")
                return await self.fail()
        else:
            return await super().message_handler(client, message)



from .pornfans import PornfansMessager

__ignore__ = True


class TestPornfansMessager(PornfansMessager):
    name = "PornFans 水群测试"
    chat_name = "api_group"
    default_messages = ["pornemby-common-wl@latest.yaml * 1000"]



import asyncio
import logging

from loguru import logger

from embykeeper.log import formatter
from embykeeper.config import config
from embykeeper.apprise import AppriseStream

debug_logger = logger.bind(scheme="debugtool")
logger = logger.bind(scheme="notifier", nonotify=True)

stream_log = None
stream_msg = None
handler_log_id = None
handler_msg_id = None
change_handle_telegram = None
change_handle_notifier = None


async def _stop_notifier():
    global stream_log, stream_msg, handler_log_id, handler_msg_id

    if handler_log_id is not None:
        logger.remove(handler_log_id)
        handler_log_id = None
    if handler_msg_id is not None:
        logger.remove(handler_msg_id)
        handler_msg_id = None

    if stream_log:
        stream_log.close()
        await stream_log.join()
        stream_log = None
    if stream_msg:
        stream_msg.close()
        await stream_msg.join()
        stream_msg = None


def _handle_config_change(*args):
    async def _async():
        global stream_log, stream_msg

        await _stop_notifier()
        if config.notifier and config.notifier.enabled:
            streams = await start_notifier()
            if streams:
                stream_log, stream_msg = streams

    logger.debug("正在刷新 Telegram 消息通知.")
    asyncio.create_task(_async())


async def start_notifier():
    """消息通知初始化函数."""
    global stream_log, stream_msg, handler_log_id, handler_msg_id, change_handle_telegram, change_handle_notifier

    def _filter_log(record):
        notify = record.get("extra", {}).get("log", None)
        nonotify = record.get("extra", {}).get("nonotify", None)
        if (not nonotify) and (notify or record["level"].no == logging.ERROR):
            return True
        else:
            return False

    def _filter_msg(record):
        notify = record.get("extra", {}).get("msg", None)
        nonotify = record.get("extra", {}).get("nonotify", None)
        if (not nonotify) and notify:
            return True
        else:
            return False

    def _formatter(record):
        return "{level}#" + formatter(record)

    notifier = config.notifier
    if not notifier or not notifier.enabled:
        if not change_handle_notifier:
            change_handle_notifier = config.on_change("notifier", _handle_config_change)
        return None

    if notifier.method == "apprise":
        if not notifier.apprise_uri:
            logger.error("Apprise URI 未配置, 无法发送消息推送.")
            return None

        logger.info("关键消息将通过 Apprise 推送.")
        stream_log = AppriseStream(uri=notifier.apprise_uri)
        handler_log_id = logger.add(
            stream_log,
            format=_formatter,
            filter=_filter_log,
            enqueue=True,
        )
        stream_msg = AppriseStream(uri=notifier.apprise_uri)
        handler_msg_id = logger.add(
            stream_msg,
            format=_formatter,
            filter=_filter_msg,
            enqueue=True,
        )
        if not change_handle_notifier:
            change_handle_notifier = config.on_change("notifier", _handle_config_change)
        return stream_log, stream_msg

    # Default to telegram
    accounts = config.telegram.account
    account = None
    if isinstance(notifier.account, int):
        try:
            account = accounts[notifier.account - 1]
        except IndexError:
            pass
    elif isinstance(notifier.account, str):
        for a in accounts:
            if a.phone == notifier.account:
                account = a
                break

    if account:
        from .telegram.session import ClientsSession
        from .telegram.log import TelegramStream

        async with ClientsSession([account]) as clients:
            async for a, tg in clients:
                logger.info(f'计划任务的关键消息将通过 Embykeeper Bot 发送至 "{account.phone}" 账号.')
                break
            else:
                logger.error(f'无法连接到 "{account.phone}" 账号, 无法发送日志推送.')
                return None

        stream_log = TelegramStream(
            account=account,
            instant=config.notifier.immediately,
        )
        handler_log_id = logger.add(
            stream_log,
            format=_formatter,
            filter=_filter_log,
        )
        stream_msg = TelegramStream(
            account=account,
            instant=True,
        )
        handler_msg_id = logger.add(
            stream_msg,
            format=_formatter,
            filter=_filter_msg,
        )
        if not change_handle_telegram:
            change_handle_telegram = config.on_change("telegram.account", _handle_config_change)
        if not change_handle_notifier:
            change_handle_notifier = config.on_change("notifier", _handle_config_change)
        return stream_log, stream_msg
    else:
        logger.error(f"无法找到消息推送所配置的 Telegram 账号.")
        if not change_handle_notifier:
            change_handle_notifier = config.on_change("notifier", _handle_config_change)
        return None


async def debug_notifier():
    streams = await start_notifier()
    if streams:
        logger.info("以下是发送的日志:")
        debug_logger.bind(msg=True).info("这是一条用于测试的即时消息, 使用 debug_notify 触发 😉.")
        debug_logger.bind(log=True).info("这是一条用于测试的日志消息, 使用 debug_notify 触发 😉.")
        if config.notifier.method == "apprise":
            logger.info("已尝试发送, 请至 Apprise 配置的接收端查看.")
        elif config.notifier.method == "telegram":
            logger.info("已尝试发送, 请至 @embykeeper_bot 查看.")
        await asyncio.gather(*[stream.join() for stream in streams if stream])
    else:
        logger.error("您当前没有配置有效的日志通知 (未启用日志通知或未配置账号), 请检查配置文件.")



import asyncio

from pyrogram.handlers import EditedMessageHandler, MessageHandler, DeletedMessagesHandler
from pyrogram.raw.base.messages.bot_callback_answer import BotCallbackAnswer

from embykeeper.runinfo import RunStatus

from ._templ_a import TemplateACheckin


__ignore__ = True


class AskCheckin(TemplateACheckin):
    name = "Ask"
    bot_username = "askUniversal_bot"
    templ_panel_keywords = ["欢迎"]
    bot_checkin_button = ["🧧"]
    bot_success_keywords = ["恭喜获得"]

    async def init(self):
        self.click_counts = 0
        return await super().init()

    async def on_button_answer(self, answer: BotCallbackAnswer):
        self.click_counts += 1
        if self.click_counts > 20:
            return await self.fail()
